from django.conf import settings
from django import get_version
from django.contrib import admin
from django.contrib import messages
from django.core.paginator import PageNotAnInteger, EmptyPage
from django.shortcuts import redirect, reverse
from django.template.response import TemplateResponse
from django.urls import path
from drf_user_activity_tracker_mongodb.utils import (MyCollection,
                                                     get_all_url_names,
                                                     ParamsHandler,
                                                     CustomPaginator,
                                                     database_log_enabled)

if database_log_enabled():
    from drf_user_activity_tracker_mongodb.models import ActivityLog


    @admin.register(ActivityLog)
    class ActivityLogAdmin(admin.ModelAdmin):
        model = ActivityLog

        def get_urls(self):
            info = "{}_{}_changelist".format(self.model._meta.app_label, self.model._meta.model_name)
            detail = "{}_{}_change".format(self.model._meta.app_label, self.model._meta.model_name)

            return [
                path(r'', self.activity_log_view, name=info),
                path(r'<str:pk>/change/', self.activity_log_detail_view, name=detail)
            ]

        def activity_log_view(self, request):
            params = ParamsHandler(request)

            url_names_list = get_all_url_names()
            url_name = params.get_url_name()
            search_value = params.get_search_value()
            status_code = params.get_status()
            time_delta = params.get_time_delta()

            data_count = MyCollection().data_count(url_name=url_name, user_id=search_value, status_code=status_code,
                                                   time_delta=time_delta)

            dataset_limit = 50
            if hasattr(settings, 'DRF_ACTIVITY_TRACKER_DJANGO_ADMIN_LIMIT'):
                if isinstance(settings.DRF_ACTIVITY_TRACKER_DJANGO_ADMIN_LIMIT, int):
                    dataset_limit = settings.DRF_ACTIVITY_TRACKER_DJANGO_ADMIN_LIMIT

            page = request.GET.get('page', "")

            if page.isdigit():
                page = int(page)
            else:
                page = 1

            skip = (page - 1) * dataset_limit

            dataset = MyCollection().list(url_name=url_name, user_id=search_value, status_code=status_code,
                                          time_delta=time_delta, dataset_limit=dataset_limit, skip=skip)

            paginator = CustomPaginator(dataset=dataset, data_count=data_count, per_page=dataset_limit)

            try:
                page_object = paginator.page(page)
            except PageNotAnInteger:
                page_object = paginator.page(1)
            except EmptyPage:
                page_object = paginator.page(paginator.num_pages)

            context = dict(self.admin_site.each_context(request), dataset=dataset, page_object=page_object,
                           url_names=url_names_list,
                           count=data_count)

            if int(get_version()[0]) == 2:
                return TemplateResponse(request, "activity_log/admin/change_list_v2.html", context=context)

            return TemplateResponse(request, "activity_log/admin/change_list_v3.html", context=context)

        def activity_log_detail_view(self, request, pk=None):
            data = MyCollection().detail(pk)
            if not data:
                messages.warning(request, f"Log with ID '{pk}' doesn’t exist. Perhaps it was deleted?")
                return redirect(reverse('admin:index'))
            context = dict(self.admin_site.each_context(request), data=data)
            return TemplateResponse(request, "activity_log/admin/change_detail.html", context=context)



from urllib.parse import urlencode

from django import template

from drf_user_activity_tracker_mongodb.utils import CustomPaginator

register = template.Library()


@register.simple_tag
def under_score_tag(obj, attribute):
    obj = dict(obj)
    return obj.get(attribute)


@register.simple_tag
def clean_url_encode(data, remove):
    attrs = data.copy()
    if remove in data:
        attrs.pop(remove)
    return urlencode(attrs)


@register.simple_tag
def get_proper_elided_page_range(p, count, number, on_each_side=3, on_ends=2):
    paginator = CustomPaginator(p.dataset, count, p.per_page)
    return paginator.get_elided_page_range(number=number,
                                           on_each_side=on_each_side,
                                           on_ends=on_ends)



from datetime import datetime

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from rest_framework.exceptions import ValidationError


class QueryParamsValidatorSerializer(serializers.Serializer):
    created_time_after = serializers.DateField(required=False)
    created_time_before = serializers.DateField(required=False)
    url_name = serializers.CharField(required=False)
    user_id = serializers.IntegerField(required=False)

    def validate(self, attrs):
        if attrs.get("created_time_after") and attrs.get("created_time_before"):
            if attrs.get("created_time_after") > attrs.get("created_time_before"):
                raise ValidationError(_("from time occurred after to time!"))

        # Mongodb does not support date field, these scripts convert date fields to datetime fields
        if attrs.get("created_time_after"):
            created_time_after_date = attrs.get("created_time_after")
            created_time_after_date_time = datetime(year=created_time_after_date.year,
                                                    month=created_time_after_date.month,
                                                    day=created_time_after_date.day)
            attrs['created_time_after'] = created_time_after_date_time

        if attrs.get("created_time_before"):
            created_time_before_date = attrs.get("created_time_before")
            created_time_before_date_time = datetime(year=created_time_before_date.year,
                                                     month=created_time_before_date.month,
                                                     day=created_time_before_date.day,
                                                     hour=23, minute=59, second=59)
            attrs['created_time_before'] = created_time_before_date_time

        return attrs


class ActivityLogSerializer(serializers.Serializer):
    _id = serializers.CharField()
    event_name = serializers.SerializerMethodField(read_only=True)
    created_time = serializers.DateTimeField()
    client_ip_address = serializers.CharField()
    url_name = serializers.CharField(max_length=255, write_only=True)

    class Meta:
        fields = ['_id', 'event_name', 'client_ip_address', 'created_time']

    def get_event_name(self, obj):
        event_name = obj.get('url_name', '')
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_EVENT_NAME'):
            if isinstance(settings.DRF_ACTIVITY_TRACKER_EVENT_NAME, dict):
                event_name = settings.DRF_ACTIVITY_TRACKER_EVENT_NAME.get(obj.get('url_name'), obj.get('url_name'))
                if not event_name:
                    event_name = obj.get('url_name', '')
        return event_name


class ActivityLogAdminSerializer(ActivityLogSerializer):
    user_id = serializers.IntegerField()

    class Meta:
        fields = ['_id', 'user_id', 'event_name', 'client_ip_address', 'created_time']



import os
from drf_user_activity_tracker_mongodb.events import Events

if os.environ.get('RUN_MAIN', None) != 'true':
    default_app_config = 'drf_user_activity_tracker_mongodb.apps.LoggerConfig'

ACTIVITY_TRACKER_SIGNAL = Events()



from rest_framework import mixins
from rest_framework.generics import GenericAPIView
from rest_framework.pagination import LimitOffsetPagination
from rest_framework.permissions import IsAuthenticated
from rest_framework_simplejwt.authentication import JWTAuthentication

from drf_user_activity_tracker_mongodb.permissions import CanViewAdminHistory
from drf_user_activity_tracker_mongodb.serializers import ActivityLogSerializer, ActivityLogAdminSerializer, \
    QueryParamsValidatorSerializer
from drf_user_activity_tracker_mongodb.utils import MyCollection, create_time_delta_for_api


class ActivityLogView(mixins.ListModelMixin, GenericAPIView):
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated]
    serializer_class = ActivityLogSerializer
    pagination_class = LimitOffsetPagination

    def get_queryset(self):
        date_validator_serializer = QueryParamsValidatorSerializer(data=self.request.query_params)
        date_validator_serializer.is_valid(raise_exception=True)

        created_time_after = date_validator_serializer.validated_data.get("created_time_after")
        created_time_before = date_validator_serializer.validated_data.get("created_time_before")
        time_delta = create_time_delta_for_api(created_time_after, created_time_before)
        if time_delta:
            return MyCollection().api_list(user_id=self.request.user.id, time_delta=time_delta)

        return MyCollection().api_list(user_id=self.request.user.id)

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class ActivityLogAdminView(mixins.ListModelMixin, GenericAPIView):
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated, CanViewAdminHistory]
    serializer_class = ActivityLogAdminSerializer
    pagination_class = LimitOffsetPagination

    def get_queryset(self):
        query_params_validator_serializer = QueryParamsValidatorSerializer(data=self.request.query_params)
        query_params_validator_serializer.is_valid(raise_exception=True)

        created_time_after = query_params_validator_serializer.validated_data.get("created_time_after")
        created_time_before = query_params_validator_serializer.validated_data.get("created_time_before")
        time_delta = create_time_delta_for_api(created_time_after, created_time_before)

        url_name = query_params_validator_serializer.validated_data.get('url_name')
        user_id = query_params_validator_serializer.validated_data.get('user_id')

        return MyCollection().api_list(time_delta=time_delta, url_name=url_name, user_id=user_id)

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)



import collections.abc
import importlib
import re
from math import ceil

from bson import ObjectId
from django.apps import apps
from django.conf import settings
from django.core.paginator import Paginator
from django.utils import timezone
from django.utils.functional import cached_property
from pymongo import MongoClient

SENSITIVE_KEYS = ['password', 'access', 'refresh']
if hasattr(settings, 'DRF_ACTIVITY_TRACKER_EXCLUDE_KEYS'):
    if isinstance(settings.DRF_ACTIVITY_TRACKER_EXCLUDE_KEYS, (list, tuple)):
        SENSITIVE_KEYS.extend(settings.DRF_ACTIVITY_TRACKER_EXCLUDE_KEYS)


def get_headers(request=None):
    """
        Function:       get_headers(self, request)
        Description:    To get all the headers from request
    """
    regex = re.compile('^HTTP_')
    return dict((regex.sub('', header), value) for (header, value)
                in request.META.items() if header.startswith('HTTP_'))


def get_client_ip(request):
    try:
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[0]
        else:
            ip = request.META.get('REMOTE_ADDR')
        return ip
    except:
        return ''


def is_activity_tracker_enabled():
    drf_activity_tracker_database = False
    if hasattr(settings, 'DRF_ACTIVITY_TRACKER_DATABASE'):
        drf_activity_tracker_database = settings.DRF_ACTIVITY_TRACKER_DATABASE

    drf_activity_tracker_signal = False
    if hasattr(settings, 'DRF_ACTIVITY_TRACKER_SIGNAL'):
        drf_activity_tracker_signal = settings.DRF_ACTIVITY_TRACKER_SIGNAL
    return drf_activity_tracker_database or drf_activity_tracker_signal


def database_log_enabled():
    drf_activity_tracker_database = False
    if hasattr(settings, 'DRF_ACTIVITY_TRACKER_DATABASE'):
        drf_activity_tracker_database = settings.DRF_ACTIVITY_TRACKER_DATABASE
    return drf_activity_tracker_database


def mask_sensitive_data(data):
    """
    Hides sensitive keys specified in sensitive_keys settings.
    Loops recursively over nested dictionaries.
    """

    if not isinstance(data, dict):
        return data

    for key, value in data.items():
        if key in SENSITIVE_KEYS:
            data[key] = "***FILTERED***"
            value = data[key]

        if isinstance(value, dict):
            data[key] = mask_sensitive_data(data[key])

        if isinstance(value, list):
            data[key] = [mask_sensitive_data(item) for item in data[key]]

    return data


class MongoConnection(object):
    def __init__(self):
        client = MongoClient(settings.DRF_ACTIVITY_TRACKER_MONGO_CONNECTION, serverSelectionTimeoutMs=5000)

        self.db = client[settings.DRF_ACTIVITY_TRACKER_MONGO_DB_NAME]
        self.collection = self.db[settings.DRF_ACTIVITY_TRACKER_MONGO_DB_COLLECTION_NAME]


class MyCollection(MongoConnection):

    def data_count(self, user_id=None, url_name=None, status_code=None, time_delta=None):
        filter_params = {}
        if user_id:
            filter_params.update({'user_id': user_id})
        if url_name:
            filter_params.update({'url_name': url_name})
        if status_code:
            filter_params.update({'status_code': {'$gte': status_code, '$lt': status_code + 100}})
        if time_delta:
            filter_params.update({'created_time': time_delta})
        return self.collection.count_documents(filter_params)

    def save(self, obj_list):
        self.collection.insert_many(obj_list)

    def list(self, user_id=None, url_name=None, status_code=None, time_delta=None, dataset_limit=0, skip=0):
        filter_params = {}
        if user_id:
            filter_params.update({'user_id': user_id})
        if time_delta:
            filter_params.update({'created_time': time_delta})
        if url_name:
            filter_params.update({'url_name': url_name})
        if status_code:
            filter_params.update({'status_code': {'$gte': status_code, '$lt': status_code + 100}})
        return list(self.collection.find(filter_params).sort('created_time', -1).limit(dataset_limit).skip(skip))

    def api_list(self, user_id=None, time_delta=None, url_name=None):
        filter_params = {}
        limit = 1500
        if hasattr(settings, 'DRF_ACTIVITI_API_LIMIT'):
            if isinstance(settings.DRF_ACTIVITI_API_LIMIT, (tuple, list)):
                limit = settings.DRF_ACTIVITI_API_LIMIT
        if user_id:
            filter_params.update({'user_id': user_id})

        if time_delta:
            filter_params.update({'created_time': time_delta})

        if url_name:
            filter_params.update({'url_name': url_name})

        if hasattr(settings, 'DRF_ACTIVITI_API_UNNECESSARY_URL_NAME'):
            if isinstance(settings.DRF_ACTIVITI_API_UNNECESSARY_URL_NAME, list):
                filter_params.update({'url_name': {'$nin': settings.DRF_ACTIVITI_API_UNNECESSARY_URL_NAME}})

        return list(self.collection.find(filter_params).sort('created_time', -1).limit(limit))

    def detail(self, pk):
        try:
            pk = ObjectId(pk)
        except:
            return None
        return self.collection.find_one({'_id': pk})


def get_all_url_names():
    list_of_all_urls = list()
    list_of_url_name = list()
    for name, app in apps.app_configs.items():
        mod_to_import = f'{name}.urls'
        try:
            urls = getattr(importlib.import_module(mod_to_import), 'urlpatterns')
            list_of_all_urls.extend(urls)
        except ImportError as ex:
            # is an app without urls
            pass
    for url in list_of_all_urls:
        if hasattr(url, 'name'):
            if url.name:
                list_of_url_name.append(url.name)

    if hasattr(settings, 'DRF_ACTIVITY_TRACKER_URL_NAMES'):
        if isinstance(settings.DRF_ACTIVITY_TRACKER_EXCLUDE_KEYS, (list, tuple)):
            list_of_url_name.extend(settings.DRF_ACTIVITY_TRACKER_URL_NAMES)

    list_of_url_name.sort()
    return list_of_url_name


class ParamsHandler:

    def __init__(self, request):
        self.params = request.GET

    def get_url_name(self):
        return self.params.get('url_name')

    def get_search_value(self):
        search_params = self.params.get('q')
        if search_params and search_params.isdigit():
            return int(search_params)

    def get_status(self):
        status_code = self.params.get('status')
        if status_code and status_code.isdigit():
            return int(status_code)

    def get_time_delta(self):
        now = timezone.now()
        today = now.today()
        return {
            'today': {'$lte': now, '$gt': today.replace(hour=0, minute=0, second=0, microsecond=0)},
            'past_7_days': {'$lte': now,
                            '$gt': today.replace(hour=0, minute=0, second=0, microsecond=0) - timezone.timedelta(
                                days=7)},
            'this_month': {'$lte': today,
                           '$gt': today.replace(day=1, hour=0, minute=0, second=0, microsecond=0)},
            'this_year': {'$lte': today,
                          '$gt': today.replace(month=1, day=1, hour=0, minute=0, second=0, microsecond=0)},
        }.get(self.params.get('created_time'))


class CustomPaginator(Paginator):
    ELLIPSIS = '…'

    def __init__(self, dataset, data_count, per_page, orphans=0,
                 allow_empty_first_page=True):
        self.data_count = data_count
        self.dataset = dataset
        self.per_page = int(per_page)
        self.orphans = int(orphans)
        self.allow_empty_first_page = allow_empty_first_page

    def page(self, number):
        """Return a Page object for the given 1-based page number."""
        number = self.validate_number(number)
        return self._get_page(self.dataset, number, self)

    @cached_property
    def count(self):
        return None

    @cached_property
    def num_pages(self):
        """Return the total number of pages."""
        if self.data_count == 0 and not self.allow_empty_first_page:
            return 0
        hits = max(1, self.data_count - self.orphans)
        return ceil(hits / self.per_page)

    def get_elided_page_range(self, number=1, *, on_each_side=3, on_ends=2):
        """
        Return a 1-based range of pages with some values elided.

        If the page range is larger than a given size, the whole range is not
        provided and a compact form is returned instead, e.g. for a paginator
        with 50 pages, if page 43 were the current page, the output, with the
        default arguments, would be:

            1, 2, …, 40, 41, 42, 43, 44, 45, 46, …, 49, 50.
        """
        number = self.validate_number(number)

        if self.num_pages <= (on_each_side + on_ends) * 2:
            yield from self.page_range
            return

        if number > (1 + on_each_side + on_ends) + 1:
            yield from range(1, on_ends + 1)
            yield self.ELLIPSIS
            yield from range(number - on_each_side, number + 1)
        else:
            yield from range(1, number + 1)

        if number < (self.num_pages - on_each_side - on_ends) - 1:
            yield from range(number + 1, number + on_each_side + 1)
            yield self.ELLIPSIS
            yield from range(self.num_pages - on_ends + 1, self.num_pages + 1)
        else:
            yield from range(number + 1, self.num_pages + 1)


class Page(collections.abc.Sequence):

    def __init__(self, dataset, number, paginator):
        self.dataset = dataset
        self.number = number
        self.paginator = paginator

    def __repr__(self):
        return '<Page %s of %s>' % (self.number, self.paginator.num_pages)

    def __len__(self):
        return len(self.dataset)

    def __getitem__(self, index):
        if not isinstance(index, (int, slice)):
            raise TypeError(
                'Page indices must be integers or slices, not %s.'
                % type(index).__name__
            )
        return self.dataset

    def has_next(self):
        return self.number < self.paginator.num_pages

    def has_previous(self):
        return self.number > 1

    def has_other_pages(self):
        return self.has_previous() or self.has_next()

    def next_page_number(self):
        return self.paginator.validate_number(self.number + 1)

    def previous_page_number(self):
        return self.paginator.validate_number(self.number - 1)

    def start_index(self):
        """
        Return the 1-based index of the first object on this page,
        relative to total objects in the paginator.
        """
        # Special case, return zero if no items.
        if self.paginator.count == 0:
            return 0
        return (self.paginator.per_page * (self.number - 1)) + 1

    def end_index(self):
        """
        Return the 1-based index of the last object on this page,
        relative to total objects found (hits).
        """
        # Special case for the last page because there can be orphans.
        if self.number == self.paginator.num_pages:
            return self.paginator.count
        return self.number * self.paginator.per_page


def create_time_delta_for_api(from_date, to_date):
    time_delta = None

    created_time_after = from_date
    created_time_before = to_date

    if created_time_after and created_time_before:
        time_delta = {'$gte': created_time_after, '$lte': created_time_before}

    if created_time_after and not created_time_before:
        time_delta = {'$gte': created_time_after}

    if created_time_before and not created_time_after:
        time_delta = {'$lte': created_time_before}

    return time_delta



import ipaddress
import json
import pathlib
import time

import jwt
import pygeoip
from django.conf import settings
from django.contrib.auth import get_user_model
from django.urls import resolve
from django.utils import timezone
from rest_framework.parsers import MultiPartParser

from drf_user_activity_tracker_mongodb import ACTIVITY_TRACKER_SIGNAL
from drf_user_activity_tracker_mongodb.start_logger_when_server_starts import LOGGER_THREAD
from drf_user_activity_tracker_mongodb.utils import get_headers, get_client_ip, mask_sensitive_data

User = get_user_model()

"""
File: activity_tracker_middleware.py
Class: ActivityTrackerMiddleware
"""


class ActivityTrackerMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

        self.DRF_ACTIVITY_TRACKER_DATABASE = False
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_DATABASE'):
            self.DRF_ACTIVITY_TRACKER_DATABASE = settings.DRF_ACTIVITY_TRACKER_DATABASE

        self.DRF_ACTIVITY_TRACKER_SIGNAL = False
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_SIGNAL'):
            self.DRF_ACTIVITY_TRACKER_SIGNAL = settings.DRF_ACTIVITY_TRACKER_SIGNAL

        self.DRF_ACTIVITY_TRACKER_PATH_TYPE = 'ABSOLUTE'
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_PATH_TYPE'):
            if settings.DRF_ACTIVITY_TRACKER_PATH_TYPE in ['ABSOLUTE', 'RAW_URI', 'FULL_PATH']:
                self.DRF_ACTIVITY_TRACKER_PATH_TYPE = settings.DRF_ACTIVITY_TRACKER_PATH_TYPE

        self.DRF_ACTIVITY_TRACKER_SKIP_URL_NAME = ['user_history', 'admin_history']
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_SKIP_URL_NAME'):
            if isinstance(settings.DRF_ACTIVITY_TRACKER_SKIP_URL_NAME, (tuple, list)):
                self.DRF_ACTIVITY_TRACKER_SKIP_URL_NAME.extend(settings.DRF_ACTIVITY_TRACKER_SKIP_URL_NAME)

        self.DRF_ACTIVITY_TRACKER_DONT_SKIP_URL_NAME = []
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_DONT_SKIP_URL_NAME'):
            if isinstance(settings.DRF_ACTIVITY_TRACKER_DONT_SKIP_URL_NAME, (tuple, list)):
                self.DRF_ACTIVITY_TRACKER_DONT_SKIP_URL_NAME.extend(settings.DRF_ACTIVITY_TRACKER_DONT_SKIP_URL_NAME) 

        self.DRF_ACTIVITY_TRACKER_SKIP_NAMESPACE = []
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_SKIP_NAMESPACE'):
            if isinstance(settings.DRF_ACTIVITY_TRACKER_SKIP_NAMESPACE, (tuple, list)):
                self.DRF_ACTIVITY_TRACKER_SKIP_NAMESPACE = settings.DRF_ACTIVITY_TRACKER_SKIP_NAMESPACE

        self.DRF_ACTIVITY_TRACKER_METHODS = []
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_METHODS'):
            if isinstance(settings.DRF_ACTIVITY_TRACKER_METHODS, (tuple, list)):
                self.DRF_ACTIVITY_TRACKER_METHODS = settings.DRF_ACTIVITY_TRACKER_METHODS

    def __call__(self, request):

        # Run only if logger is enabled.
        if self.DRF_ACTIVITY_TRACKER_DATABASE or self.DRF_ACTIVITY_TRACKER_SIGNAL:

            url_name = resolve(request.path).url_name
            namespace = resolve(request.path).namespace

            # Always skip Admin panel
            if namespace == 'admin':
                return self.get_response(request)

            # Skip for url name
            if url_name in self.DRF_ACTIVITY_TRACKER_SKIP_URL_NAME:
                return self.get_response(request)

            # Skip entire app using namespace
            if namespace in self.DRF_ACTIVITY_TRACKER_SKIP_NAMESPACE:
                return self.get_response(request)

            start_time = time.time()
            request_data = ''
            try:
                if request.content_type in ('application/json', 'application/vnd.api+json',):
                    request_data = json.loads(request.body) if request.body else ''
                elif request.content_type == 'multipart/form-data':
                    parser_obj = MultiPartParser()
                    context = {'request': request}
                    request_data = parser_obj.parse(stream=request._stream, media_type=request.META['CONTENT_TYPE'],
                                                    parser_context=context) if request.body else ''
                    request_data = dict(request_data.data)
            except:
                pass

            # Code to be executed for each request before
            # the view (and later middleware) are called.
            response = self.get_response(request)

            # Code to be executed for each request/response after
            # the view is called.

            header_token = request.META.get("HTTP_AUTHORIZATION")
            algorithm = 'HS256'
            if hasattr(settings, "JWT_ALGORITHM") and isinstance(settings.JWT_ALGORITHM, str):
                algorithm = settings.JWT_ALGORITHM

            if header_token:
                try:
                    token = header_token.split()[1]
                    user_token = jwt.decode(jwt=token, key=settings.SECRET_KEY, algorithms=[algorithm])
                    user = User.objects.get(id=user_token.get('user_id'))
                except:
                    return response
            elif hasattr(response, 'data') and isinstance(response.data, dict) and response.data.get('access'):
                user_token = jwt.decode(jwt=response.data.get('access'), key=settings.SECRET_KEY,
                                        algorithms=[algorithm])
                try:
                    user = User.objects.get(id=user_token.get('user_id'))
                except User.DoesNotExist:
                    return response
            elif url_name in self.DRF_ACTIVITY_TRACKER_DONT_SKIP_URL_NAME:
                user = None
            else:
                return response

            headers = get_headers(request=request)
            method = request.method

            # Log only registered methods if available.
            if len(self.DRF_ACTIVITY_TRACKER_METHODS) > 0 and method not in self.DRF_ACTIVITY_TRACKER_METHODS:
                return self.get_response(request)

            if response.get('content-type') in ('application/json', 'application/vnd.api+json',):
                if getattr(response, 'streaming', False):
                    response_body = '** Streaming **'
                else:
                    if isinstance(response.content, bytes):
                        response_body = json.loads(response.content.decode())
                    else:
                        response_body = json.loads(response.content)
                if self.DRF_ACTIVITY_TRACKER_PATH_TYPE == 'ABSOLUTE':
                    api = request.build_absolute_uri()
                elif self.DRF_ACTIVITY_TRACKER_PATH_TYPE == 'FULL_PATH':
                    api = request.get_full_path()
                elif self.DRF_ACTIVITY_TRACKER_PATH_TYPE == 'RAW_URI':
                    api = request.get_raw_uri()
                else:
                    api = request.build_absolute_uri()
                ip = get_client_ip(request)
                if ipaddress.ip_address(ip).version == 4:
                    try:
                        country_name = pygeoip.GeoIP(
                            '{}/geo_databases/GeoIP.dat'.format(pathlib.Path(__file__).parent.parent),
                            pygeoip.MEMORY_CACHE).country_name_by_addr(ip)
                    except:
                        country_name = ''
                else:
                    try:
                        country_name = pygeoip.GeoIP(
                            '{}/geo_databases/GeoIPv6.dat'.format(pathlib.Path(__file__).parent.parent),
                            pygeoip.MEMORY_CACHE).country_name_by_addr(ip)
                    except:
                        country_name = ''
                data = dict(
                    url_name=url_name,
                    url_path=request.path,
                    user_id=user.id if user else None,
                    api=api,
                    headers=mask_sensitive_data(headers),
                    body=mask_sensitive_data(request_data),
                    method=method,
                    client_ip_address=ip,
                    response=mask_sensitive_data(response_body),
                    status_code=response.status_code,
                    execution_time=time.time() - start_time,
                    created_time=timezone.now(),
                    country=country_name,
                )
                if self.DRF_ACTIVITY_TRACKER_DATABASE:
                    if LOGGER_THREAD:
                        LOGGER_THREAD.put_log_data(data=data)
                if self.DRF_ACTIVITY_TRACKER_SIGNAL:
                    ACTIVITY_TRACKER_SIGNAL.listen(**data)
            else:
                return response

        else:
            response = self.get_response(request)
        return response



class EventsException(Exception):
    pass


class Events:
    """
    Encapsulates the core to event subscription and event firing, and feels
    like a "natural" part of the language.
    The class Events is there mainly for 3 reasons:
        - Events (Slots) are added automatically, so there is no need to
        declare/create them separately. This is great for prototyping. (Note
        that `__events__` is optional and should primarily help detect
        misspelled event names.)
        - To provide (and encapsulate) some level of introspection.
        - To "steel the name" and hereby remove unneeded redundancy in a call
        like:
            xxx.OnChange = event('OnChange')
    """

    def __init__(self, events=None):

        if events is not None:

            try:
                iter(events)
            except:
                raise AttributeError("type object %s is not iterable" %
                                     (type(events)))
            else:
                self.__events__ = events

    def __getattr__(self, name):
        if name.startswith('__'):
            raise AttributeError("type object '%s' has no attribute '%s'" %
                                 (self.__class__.__name__, name))

        if hasattr(self, '__events__'):
            if name not in self.__events__:
                raise EventsException("Event '%s' is not declared" % name)

        elif hasattr(self.__class__, '__events__'):
            if name not in self.__class__.__events__:
                raise EventsException("Event '%s' is not declared" % name)

        self.__dict__[name] = ev = _EventSlot(name)
        return ev

    def __repr__(self):
        return '<%s.%s object at %s>' % (self.__class__.__module__,
                                         self.__class__.__name__,
                                         hex(id(self)))

    __str__ = __repr__

    def __len__(self):
        return len(self.__dict__.items())

    def __iter__(self):
        def gen(dictitems=self.__dict__.items()):
            for attr, val in dictitems:
                if isinstance(val, _EventSlot):
                    yield val

        return gen()


class _EventSlot:
    def __init__(self, name):
        self.targets = []
        self.__name__ = name

    def __repr__(self):
        return "event '%s'" % self.__name__

    def __call__(self, *a, **kw):
        for f in tuple(self.targets):
            f(*a, **kw)

    def __iadd__(self, f):
        self.targets.append(f)
        return self

    def __isub__(self, f):
        while f in self.targets:
            self.targets.remove(f)
        return self

    def __len__(self):
        return len(self.targets)

    def __iter__(self):
        def gen():
            for target in self.targets:
                yield target

        return gen()

    def __getitem__(self, key):
        return self.targets[key]



from drf_user_activity_tracker_mongodb.utils import database_log_enabled

LOGGER_THREAD = None

if database_log_enabled():
    from drf_user_activity_tracker_mongodb.insert_log_into_database import InsertLogIntoDatabase
    import threading

    LOG_THREAD_NAME = 'insert_log_into_database'

    already_exists = False

    for t in threading.enumerate():
        if t.getName() == LOG_THREAD_NAME:
            already_exists = True

    if not already_exists:
        t = InsertLogIntoDatabase()
        t.daemon = True
        t.setName(LOG_THREAD_NAME)
        t.start()
        LOGGER_THREAD = t



import logging
import time
from queue import Queue
from threading import Thread

from django.conf import settings

from drf_user_activity_tracker_mongodb.utils import MyCollection

logger = logging.getLogger('error')


class InsertLogIntoDatabase(Thread):

    def __init__(self):
        super().__init__()

        self.DRF_ACTIVITY_TRACKER_QUEUE_MAX_SIZE = 50  # Default queue size 50
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_QUEUE_MAX_SIZE'):
            self.DRF_ACTIVITY_TRACKER_QUEUE_MAX_SIZE = settings.DRF_ACTIVITY_TRACKER_QUEUE_MAX_SIZE

        if self.DRF_ACTIVITY_TRACKER_QUEUE_MAX_SIZE < 1:
            raise Exception("""
            DRF ACTIVITY TRACKER EXCEPTION
            Value of DRF_ACTIVITY_TRACKER_QUEUE_MAX_SIZE must be greater than 0
            """)

        self.DRF_ACTIVITY_TRACKER_INTERVAL = 10  # Default DB insertion interval is 10 seconds.
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_INTERVAL'):
            self.DRF_ACTIVITY_TRACKER_INTERVAL = settings.DRF_ACTIVITY_TRACKER_INTERVAL

            if self.DRF_ACTIVITY_TRACKER_INTERVAL < 1:
                raise Exception("""
                DRF ACTIVITY TRACKER EXCEPTION
                Value of DRF_ACTIVITY_TRACKER_INTERVAL must be greater than 0
                """)

        self._queue = Queue(maxsize=self.DRF_ACTIVITY_TRACKER_QUEUE_MAX_SIZE)

    def run(self) -> None:
        self.start_queue_process()

    def put_log_data(self, data):
        self._queue.put(data)

        if self._queue.qsize() >= self.DRF_ACTIVITY_TRACKER_QUEUE_MAX_SIZE:
            self._start_bulk_insertion()

    def start_queue_process(self):
        while True:
            time.sleep(self.DRF_ACTIVITY_TRACKER_INTERVAL)
            self._start_bulk_insertion()

    def _start_bulk_insertion(self):
        bulk_item = []
        while not self._queue.empty():
            bulk_item.append(self._queue.get())
        if bulk_item:
            self._insert_into_data_base(bulk_item)

    def _insert_into_data_base(self, bulk_item):
        try:
            MyCollection().save(bulk_item)
        except Exception as e:
            message = "DRF ACTIVITY TRACKER EXCEPTION: {}, {}".format(str(e), type(e))
            logger.error(message)
            raise Exception(message)



import importlib
import json

from django.apps import apps
from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'get all url names'

    def handle(self, *args, **options):

        list_of_all_urls = list()
        for name, app in apps.app_configs.items():
            mod_to_import = f'{name}.urls'
            try:
                urls = getattr(importlib.import_module(mod_to_import), 'urlpatterns')
                list_of_all_urls.extend(urls)
            except ImportError as ex:
                # is an app without urls
                pass
        for url in list_of_all_urls:
            if hasattr(url, 'name'):
                settings.DRF_ACTIVITY_TRACKER_EVENT_NAME.update({url.name: ''})

        result = json.dumps(settings.DRF_ACTIVITY_TRACKER_EVENT_NAME, indent=3)
        self.stdout.write(self.style.SUCCESS(result))



from rest_framework.permissions import BasePermission
from django.conf import settings


class CanViewAdminHistory(BasePermission):
    """
    Allows admins access to pass this.
    """
    DRF_ACTIVITY_TRACKER_PERMISSION = "drf_user_activity_tracker_mongodb.view_activitylog"  # Default Permission
    if hasattr(settings, 'DRF_ACTIVITY_TRACKER_PERMISSION'):
        if isinstance(settings.DRF_ACTIVITY_TRACKER_PERMISSION, str):
            DRF_ACTIVITY_TRACKER_PERMISSION = settings.DRF_ACTIVITY_TRACKER_PERMISSION

    message = 'You do not have permission to view admin history!'
    view_name = DRF_ACTIVITY_TRACKER_PERMISSION

    def has_permission(self, request, view):
        return request.user.has_perm(self.view_name)



from django.urls import path
from drf_user_activity_tracker_mongodb.views import ActivityLogView, ActivityLogAdminView

app_name = "activity_log"


urlpatterns = [
    path('user-history/', ActivityLogView.as_view(), name='user_history'),
    path('admin-history/', ActivityLogAdminView.as_view(), name='admin_history'),
]


